package be.iminds.ilabt.jfed.ui.javafx.am_list_gui;

import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import com.cathive.fx.guice.FXMLComponent;
import javafx.application.Platform;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.ListProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.control.cell.ComboBoxTableCell;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.layout.BorderPane;
import javafx.util.StringConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * ExecuteServicesPanel
 */
@FXMLComponent
public class UrlsPanel extends BorderPane {

    private static final Logger LOG = LoggerFactory.getLogger(UrlsPanel.class);
    private static ObservableList<ServerType.GeniServerRole> allRoles =
            FXCollections.observableArrayList(ServerType.GeniServerRole.values());
    @FXML
    protected Button addButton;
    @FXML
    protected TableView<AuthorityInfo.AuthorityUrl> table;

    private ListProperty<AuthorityInfo.AuthorityUrl> authorityUrls =
            new SimpleListProperty<>(FXCollections.<AuthorityInfo.AuthorityUrl>observableArrayList());
    private BooleanProperty editable = new SimpleBooleanProperty(true);

    UrlsPanel() {
    }

    public BooleanProperty editableProperty() {
        return editable;
    }

    public boolean getEditable() {
        return editable.get();
    }

    public void setEditable(boolean edit) {
        editable.set(edit);
    }

    public ListProperty<AuthorityInfo.AuthorityUrl> authorityUrlsProperty() {
        return authorityUrls;
    }

    @FXML
    protected void addCommand() {
        try {
            AuthorityInfo.AuthorityUrl newAuthInfo =
                    new AuthorityInfo.AuthorityUrl(new ServerType(ServerType.GeniServerRole.AM, 2), new URL("https://example.com/xmlrpc/am2/"));
            authorityUrls.add(newAuthInfo);
        } catch (MalformedURLException e) {
            throw new RuntimeException("This should never happen", e);
        }
    }

    @FXML
    protected void removeSelectedCommand() {
        authorityUrls.removeAll(table.getSelectionModel().getSelectedItems());
    }

    @FXML
    private void initialize() {
        //        table.editableProperty().bind(editable);
        table.setEditable(true);

        addButton.visibleProperty().bind(editable);

        TableColumn<AuthorityInfo.AuthorityUrl, ServerType.GeniServerRole> selecttypeCol = new TableColumn<>("Role");
        selecttypeCol.setCellFactory(authorityUrlGeniServerRoleTableColumn -> new SelectTypeCell());
        selecttypeCol.setCellValueFactory(authUrl -> authUrl.getValue().roleProperty());
        table.getColumns().addAll(selecttypeCol);

        TableColumn<AuthorityInfo.AuthorityUrl, String> versionCol = new TableColumn<>("version");
        versionCol.setCellValueFactory(p -> p.getValue().versionProperty());
        try {
            versionCol.setCellFactory(TextFieldTableCell.forTableColumn());
        } catch (Exception e) {
            e.printStackTrace();
        }
        versionCol.setOnEditCommit(
                t -> {
                    AuthorityInfo.AuthorityUrl authUrl =
                            t.getTableView().getItems().get(t.getTablePosition().getRow());
                    authUrl.versionProperty().set(t.getNewValue());
                }
        );
        table.getColumns().add(versionCol);


        TableColumn<AuthorityInfo.AuthorityUrl, URL> urlCol = new TableColumn<>("url");
        urlCol.setCellValueFactory(p -> p.getValue().urlProperty());
        urlCol.setCellFactory(TextFieldTableCell.forTableColumn(new StringConverter<URL>() {
            @Override
            public String toString(URL object) {
                if (object != null)
                    return object.toExternalForm();
                else
                    return "";

            }

            @Override
            public URL fromString(String string) {
                try {
                    return new URL(string);
                } catch (MalformedURLException e) {
                    LOG.warn("Could not parse given URL {}", string);
                    return null;
                }
            }
        }));
        urlCol.setOnEditCommit(
                t -> {
                    if (t.getNewValue() != null) {
                        AuthorityInfo.AuthorityUrl authUrl =
                                t.getTableView().getItems().get(t.getTablePosition().getRow());
                        authUrl.urlProperty().set(t.getNewValue());
                    } else {
                        Platform.runLater(() -> JFDialogs.create()
                                .message("Please enter a valid URL")
                                .showWarning());

                    }
                }
        );
        table.getColumns().add(urlCol);


        TableColumn<AuthorityInfo.AuthorityUrl, Object> removeCol = new TableColumn<>("");
        removeCol.visibleProperty().bind(editable);
        removeCol.setCellFactory(o -> new RemoveButtonCell());
        table.getColumns().addAll(removeCol);


        //quick hack to get somewhat usefull widths  TODO automatically set correct widths
        selecttypeCol.prefWidthProperty().bind(table.widthProperty().divide(4));
        versionCol.prefWidthProperty().bind(table.widthProperty().divide(8));
        urlCol.prefWidthProperty().bind(table.widthProperty().divide(2));
        removeCol.prefWidthProperty().bind(table.widthProperty().divide(8));


        authorityUrls.addListener((observableValue, oldUrls, newUrls) -> table.setItems(newUrls));
    }

    class URLEditingCell extends TableCell<AuthorityInfo.AuthorityUrl, URL> {
        private TextField textField;

        public URLEditingCell() {
        }

        @Override
        public void startEdit() {
            if (!isEmpty()) {
                super.startEdit();
                createTextField();
                setText(null);
                setGraphic(textField);
                textField.selectAll();
            }
        }

        @Override
        public void cancelEdit() {
            super.cancelEdit();

            setText(getItem().toString().trim());
            setGraphic(null);
        }

        @Override
        public void updateItem(URL item, boolean empty) {
            super.updateItem(item, empty);

            if (empty) {
                setText(null);
                setGraphic(null);
            } else {
                if (isEditing()) {
                    if (textField != null) {
                        textField.setText(getString().trim());
                    }
                    setText(null);
                    setGraphic(textField);
                } else {
                    setText(getString().trim());
                    setGraphic(null);
                }
            }
        }

        private void createTextField() {
            textField = new TextField(getString().trim());
            textField.editableProperty().bind(editable);
            textField.setMinWidth(this.getWidth() - this.getGraphicTextGap() * 2);
            textField.focusedProperty().addListener((arg0, arg1, arg2) -> {
                if (!arg2) {
                    URL parsedUrl = null;
                    try {
                        parsedUrl = new URL(textField.getText().trim());
                    } catch (MalformedURLException ignored) {
                    }

                    textField.getStyleClass().removeAll("validation-error", "validation-warning");
                    if (parsedUrl == null) {
                        //invalid URL, commit nothing
                        //TODO provide visual feedback
                        textField.getStyleClass().add("validation-error");
                    } else
                        commitEdit(parsedUrl);
                }
            });
        }

        private String getString() {
            return getItem() == null ? "" : getItem().toString().trim();
        }
    }

    class RemoveButtonCell extends TableCell<AuthorityInfo.AuthorityUrl, Object> {
        private Button button;

        public RemoveButtonCell() {
            setEditable(false);
            button = new Button("X");
            button.getStyleClass().add("deleteIco");
            button.visibleProperty().bind(editable);
            button.setOnAction(actionEvent -> authorityUrls.remove(getTableRow().getItem()));
            setText(null);
            if (getTableRow() == null || getTableRow().getItem() == null)
                setGraphic(null);
            else
                setGraphic(button);
        }

        @Override
        public void updateItem(Object item, boolean empty) {
            super.updateItem(item, empty);
            if (getTableRow() == null || getTableRow().getItem() == null)
                setGraphic(null);
            else
                setGraphic(button);
        }
    }

    class SelectTypeCell extends ComboBoxTableCell<AuthorityInfo.AuthorityUrl, ServerType.GeniServerRole> {
        public SelectTypeCell() {
            super(allRoles);
        }

        @Override
        public void updateItem(ServerType.GeniServerRole item, boolean empty) {
            super.updateItem(item, empty);
        }
    }
}
