package be.iminds.ilabt.jfed.lowlevel.ssh_key_info;

import be.iminds.ilabt.jfed.util.IOUtils;
import be.iminds.ilabt.jfed.util.KeyUtil;
import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpFile;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * BasicSshKeyInfo: SshKeyInfo with keys given in constructor
 * <p/>
 * typically just used to get keys, not typically used for files.
 * <p/>
 * does not support PuTTY
 */
public abstract class AbstractSshKeyInfo implements SshKeyInfo {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(AbstractSshKeyInfo.class);


    protected final boolean ensureMatchingPubFileEnabled = false;
    private static Set<File> matchingPubKeyfiles = new HashSet<File>();

    /**
     * make sure matching .pub file exists
     */
    protected void ensureMatchingPubFile(TmpFile privateKeyFile) {
        if (!ensureMatchingPubFileEnabled) return;

        File matchingPubFile = new File(privateKeyFile.getDir(), privateKeyFile.getFile().getName()+ ".pub");
        if (!matchingPubFile.exists()) {
            String publicKeyFileContent = KeyUtil.publicKeyToOpenSshAuthorizedKeysFormat(getPublicKey());
            IOUtils.stringToFile(matchingPubFile, publicKeyFileContent);
            try {
                IOUtils.assureUserOnlyPerms(matchingPubFile, true, false, false);
            } catch (IOException e) {
                LOG.warn("Could not change file permissions: " + e.getMessage(), e);
            }

            matchingPubKeyfiles.add(matchingPubFile);
            matchingPubFile.deleteOnExit();
        }
    }
}
