package be.iminds.ilabt.jfed.ui.javafx.probe_gui;

import be.iminds.ilabt.jfed.BaseGuiModule;
import be.iminds.ilabt.jfed.UncaughtGuiExceptionLogger;
import be.iminds.ilabt.jfed.highlevel.HighLevelModule;
import be.iminds.ilabt.jfed.highlevel.LowLevelModule;
import be.iminds.ilabt.jfed.highlevel.UserLoginModelModule;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.ssh_terminal_tool.unix.SshAgentHelper;
import be.iminds.ilabt.jfed.ui.javafx.userlogin.UserLoginFactory;
import be.iminds.ilabt.jfed.ui.javafx.util.ExtraInfoCallBackGUIs;
import com.cathive.fx.guice.GuiceApplication;
import com.google.inject.Module;
import javafx.application.Platform;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.util.List;

public class JFedProbeStandalone extends GuiceApplication {
    private static final Logger LOG = LoggerFactory.getLogger(JFedProbeStandalone.class);

    /* solution for MAC OS HeadlessException bug. See https://ibcn-jira.intec.ugent.be/browse/FEDIBBTDEV-239 */
    static {
        System.setProperty("java.awt.headless", "false");
    }

    @Inject
    private UserLoginFactory userLoginFactory;

    @Inject
    private ProbeFactory probeFactory;

    @Inject
    private TaskThread tt;


    @Override
    public void start(final Stage stage) throws Exception {
        Thread.setDefaultUncaughtExceptionHandler(UncaughtGuiExceptionLogger.getInstance(stage));

        ExtraInfoCallBackGUIs.prepareExtraInfoCallback(stage);

        stage.setOnCloseRequest(event -> exit());

        userLoginFactory.showUserLogin(false, stage, successfulLogin -> probeFactory.showProbe(stage));
    }

    /**
     * The main() method is ignored in correctly deployed JavaFX application.
     * main() serves only as fallback in case the application can not be
     * launched through deployment artifacts, e.g., in IDEs with limited FX
     * support. NetBeans ignores main().
     *
     * @param args the command line arguments
     */
    public static boolean debugEnabled = false;

    public static void main(String[] args) {
        for (String arg : args) {
            if (arg.equalsIgnoreCase("debug") || arg.equalsIgnoreCase("--debug") || arg.equalsIgnoreCase("-debug"))
                debugEnabled = true;
        }
        launch(args);
    }

    @Override
    public void init(List<Module> modules) throws Exception {
        modules.add(new HighLevelModule());
        modules.add(new LowLevelModule());
        modules.add(new UserLoginModelModule());
        modules.add(new BaseGuiModule());
    }

    public void exit() {
        LOG.info("Close was requested. Trying to terminate");
        tt.requestStop();

        //stop any running ssh-agent
        SshAgentHelper.stopSshAgent();

        Platform.exit();
        System.exit(0);
    }

}
